# TODO: Basic class for converting a java.util.List of RTimeSeries (a java type of the 
#		ECB_WS packag) into a R list of timeseries
# 
# Author: e922480
###############################################################################

# convert a java List<String>
convertStringList <- function (javaList) {
	rList = as.list(javaList)
	numOfStrings = length(rList)
	result = list()
	if( numOfStrings != 0 ) {
		for (i in 1:numOfStrings)  {
			result[[i]] = .jstrVal(rList[[i]])
		}
	}
	return(result)
}

# convert the structure
convertDataFlowStructure <- function (structure) {
  jdims = .jcall(structure,"Ljava/util/List;","getDimensions");
  dims = convertStringList(jdims)
  names(dims)<-dims
  for(x in names(dims)){
    jcodes = .jcall(structure,"Ljava/util/List;","getCodeLists", x);
    codes = convertStringList(jcodes)
    dims[[x]]=codes
  }
  
  return(dims)
}

# convert a java HashTable<String, String>
convertHashTable <- function (javaList) {
	idSet = .jcall(javaList,"Ljava/util/Set;","keySet");
	ids = .jcall(idSet,"[Ljava/lang/Object;","toArray");
	rIDs = as.list(ids)
	
	nameSet = .jcall(javaList,"Ljava/util/Collection;","values");
	names = .jcall(nameSet,"[Ljava/lang/Object;","toArray");
	rNames = as.list(names)
	
	numOfIDs = length(rIDs)
	result = list()
	if( numOfIDs != 0 ) {
		for (i in 1:numOfIDs)  {
			id = .jstrVal(rIDs[[i]]);
			result[[id]] = .jstrVal(rNames[[i]])
#			print(paste("Found ", id, " ", result[[id]]));
		}
	}
	return(result)
}

# convert a java List<PortableTimeSeries>
convertTSList <- function (javaList) {

	rList = as.list(javaList);
	numOfTS = length(rList);
	#print(paste("Found ", numOfTS, " timeseries"));
	result = list();
	if( numOfTS > 0 ) {
		for (i in 1:numOfTS)  {
			s = .jcast(rList[[i]], new.class = "it/bankitalia/reri/sia/sdmxws/api/PortableTimeSeries", check = TRUE);
			name = .jcall(s,"Ljava/lang/String;","getName", evalString = TRUE);
			#print(paste("Name: ", name));
			freq = .jcall(s,"Ljava/lang/String;","getFrequency", evalString = TRUE);
			#print(paste("Frequency: ", freq));
			dimensions = .jcall(s,"[Ljava/lang/String;","getDimensionsArray", evalArray = TRUE, 
					evalString = TRUE);
			#print(paste("Dimensions: ", dimensions));
			attributes = .jcall(s,"[Ljava/lang/String;","getAttributesArray", evalArray = TRUE, 
					evalString = TRUE);
			#print(paste("Attributes: ", attributes));
			timeSlots = .jcall(s,"[Ljava/lang/String;","getTimeSlotsArray", evalArray = TRUE, 
					evalString = TRUE);
			#print(paste("timeSlots: ", timeSlots));
			observationsJ = .jcall(s,"[Ljava/lang/Double;","getObservationsArray", evalArray = TRUE);
			status = .jcall(s,"[Ljava/lang/String;","getStatusArray", evalArray = TRUE, 
					evalString = TRUE);
			numOfObs = length(observationsJ);
			numOfTimes = length(timeSlots);
			if( numOfObs > 0 && numOfObs == numOfTimes){
				observations = array();
				# print(paste("numOfObs: ", numOfObs))
				if(numOfObs != 0){
					for(k in 1:numOfObs){
						observations[k] = .jcall(observationsJ[[k]],"D","doubleValue");
					}
				}
				timeSeries = makeSDMXts(name, freq, timeSlots, observations, attributes, dimensions, status);
				result[[name]] = timeSeries;
			}
			else{
				print(paste("Number of observations and time slots equal to zero, or not matching: ", numOfObs, " ", numOfTimes));
			}
		}
	}
	return(result);
}

# parameters used:
# Tsname = name to be given to the series
# freq = sdmx frequency
# times= array of dates
# values= array of values
# series_attr_names= list of names of series attributes
# series_attr_values= list of values of series attributes
# status= list of values of status attribute
# convert the frequency from SDMX-like codelist to numeric, e.g. 'M'-> 12 etc..

makeSDMXts<- function (TSname,freq,times,values,series_attr, series_dims, status) {
	
	if(length(values > 0)) {

		if (is.null(freq)) {
			print ("Frequency is NULL. Irregular timeseries defined")	
			tmp_ts <- zoo(values, order.by=as.Date(times))
		} 
		else {
      if(freq == 'A'){
        tmp_ts<- zoo(values, order.by = as.integer(times),frequency=1)
      } else if(freq == 'M'){
        tmp_ts<- zoo(values, order.by = as.yearmon(times),frequency=12)
      } else if(freq == 'Q'){
        tmp_ts<- zoo(values, order.by = as.yearqtr(sub(pattern='-', replacement=' ', times)),frequency=4)
      } else {
        print (paste('Frequency ',freq, '. Does not allow the creation of a strictly regular time series. Irregular timeseries defined', sep=''))
	tmp_ts <- zoo(values, order.by=as.Date(times))
      }
	}
	}
	else{
		# if the time series is empty
		tmp_ts <- zoo()
	}
	
	# define ts attributes
	attr(tmp_ts,"Id") <- TSname
	
	# define ts attributes
	attr(tmp_ts,"STATUS") <- status
	
	# define the timeseries attributes (mimensions + attributes)
	#
	if (length(series_dims) >0){
		for (l in 1:length(series_dims)) {
			attrComp=(unlist(strsplit(series_dims[l], "=", fixed = TRUE)))
			attr(tmp_ts,attrComp[1]) <- (attrComp[2])
		}
	}
	if (length(series_attr) >0){
		for (l in 1:length(series_attr)) {
			attrComp=(unlist(strsplit(series_attr[l], "=", fixed = TRUE)))
			attr(tmp_ts,attrComp[1]) <- (attrComp[2])
		}
	}
	return(tmp_ts)
	
}

